package ru.dbotthepony.mc.otm.network

import net.minecraft.network.FriendlyByteBuf
import net.minecraft.resources.ResourceLocation
import net.minecraft.server.level.ServerPlayer
import net.minecraft.world.entity.Entity
import net.minecraft.world.entity.player.Player
import net.minecraftforge.network.NetworkDirection
import net.minecraftforge.network.NetworkEvent
import net.minecraftforge.network.NetworkRegistry
import net.minecraftforge.network.PacketDistributor
import net.minecraftforge.network.simple.SimpleChannel
import org.apache.logging.log4j.LogManager
import ru.dbotthepony.kommons.util.Delegate
import ru.dbotthepony.kommons.util.getValue
import ru.dbotthepony.kommons.util.setValue
import ru.dbotthepony.mc.otm.NULLABLE_MINECRAFT_SERVER
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import java.util.concurrent.CompletableFuture
import java.util.concurrent.ConcurrentLinkedQueue
import java.util.concurrent.locks.LockSupport
import java.util.function.BiConsumer
import java.util.function.Function
import java.util.function.Supplier
import kotlin.reflect.KClass

class MNetworkContext(val sender: ServerPlayer?, packetHandled: Delegate<Boolean>, private val enqueuer: (Runnable) -> CompletableFuture<*>) {
	var packetHandled by packetHandled

	fun enqueueWork(callback: Runnable): CompletableFuture<*> {
		return enqueuer(callback)
	}
}

interface MatteryPacket {
	fun write(buff: FriendlyByteBuf)
	fun play(context: MNetworkContext)
}

abstract class MatteryNetworkChannel(val version: Int, val name: String) {
	val channel: SimpleChannel = NetworkRegistry
		.newSimpleChannel(ResourceLocation(OverdriveThatMatters.MOD_ID, name), { version.toString() }, { it == NetworkRegistry.ACCEPTVANILLA || it == NetworkRegistry.ABSENT.version || it == version.toString() }, { it == version.toString() })

	fun sendToServer(packet: Any) = channel.send(PacketDistributor.SERVER.noArg(), packet)

	fun send(ply: Player, packet: Any) {
		if (ply is ServerPlayer) {
			channel.send(PacketDistributor.PLAYER.with { ply }, packet)
		}
	}

	fun sendTracking(entity: Entity, packet: Any) {
		if ((NULLABLE_MINECRAFT_SERVER?.playerCount ?: 0) <= 0) {
			return
		}

		channel.send(PacketDistributor.TRACKING_ENTITY.with { entity }, packet)
	}

	fun sendTrackingAndSelf(entity: Entity, packet: Any) {
		if ((NULLABLE_MINECRAFT_SERVER?.playerCount ?: 0) <= 0) {
			return
		}

		channel.send(PacketDistributor.TRACKING_ENTITY_AND_SELF.with { entity }, packet)
	}

	fun send(distributor: PacketDistributor.PacketTarget, packet: Any) {
		channel.send(distributor, packet)
	}

	private var nextNetworkPacketID = 0

	fun <T> add(
		packetClass: Class<T>,
		writer: BiConsumer<T, FriendlyByteBuf>,
		reader: Function<FriendlyByteBuf, T>,
		handler: (T, MNetworkContext) -> Unit,
		direction: NetworkDirection? = null,
		handleOnMainThread: Boolean = true,
	) {
		if (nextNetworkPacketID >= 256) {
			throw IndexOutOfBoundsException("Network message ID overflow!")
		}

		val builder = channel.messageBuilder(packetClass, nextNetworkPacketID++)
		val bridgeHandler = BiConsumer<T, Supplier<NetworkEvent.Context>> { a, b -> handler(a, MNetworkContext(b.get().sender, Delegate.Of({ b.get().packetHandled }, { b.get().packetHandled = it }), b.get()::enqueueWork)) }

		if (handleOnMainThread) {
			builder.consumerMainThread(bridgeHandler)
		} else {
			builder.consumerNetworkThread(bridgeHandler)
		}

		builder.encoder(writer)
		builder.decoder(reader)
		builder.add()
	}

	fun <T : MatteryPacket> add(
		packetClass: Class<T>,
		reader: Function<FriendlyByteBuf, T>,
		direction: NetworkDirection? = null,
		handleOnMainThread: Boolean = true,
	) {
		add(packetClass, MatteryPacket::write, reader, MatteryPacket::play, direction, handleOnMainThread)
	}

	fun <T : MatteryPacket> add(
		packetClass: KClass<T>,
		reader: Function<FriendlyByteBuf, T>,
		direction: NetworkDirection? = null,
		handleOnMainThread: Boolean = true,
	) {
		add(packetClass.java, MatteryPacket::write, reader, MatteryPacket::play, direction, handleOnMainThread)
	}
}
